package biz.hammurapi.antlr;

import java.util.ArrayList;
import java.util.List;

import antlr.collections.AST;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeNode;

import biz.hammurapi.swing.LazyTreeNode;
import biz.hammurapi.swing.TableVisualizable;

public class AstVisualizable extends LazyTreeNode implements TableVisualizable {
	
	private AST ast;
	private String[] tokenNames;
	private boolean withSiblings;

	public AstVisualizable(TreeNode parent, AST ast, String[] names, boolean withSiblings) {
		super(parent, withSiblings? "Syntax Tree" : names[ast.getType()]);
		this.ast = ast;
		tokenNames = names;
		this.withSiblings = withSiblings;
	}
	
	protected List loadChildren() {
		List ret = new ArrayList();
		for (AST child= withSiblings ? ast : ast.getFirstChild(); child!=null; child= child.getNextSibling()) {
			ret.add(new AstVisualizable(this, child, tokenNames, false));
		}
		return ret;
	}
	
	public TableModel toTable() {
		DefaultTableModel ret=new DefaultTableModel(4, 2);
		
		ret.setColumnIdentifiers(new String[] {"Property", "Value"});
		setRow(ret, 0, "Type", tokenNames[ast.getType()]);
		setRow(ret, 1, "Text", ast.getText());
		setRow(ret, 2, "Line", String.valueOf(ast.getLine()));
		setRow(ret, 3, "Column", String.valueOf(ast.getColumn()));
		
		return ret;
	}

	private static void setRow(DefaultTableModel ret, int row, String name, Object value) {
		ret.setValueAt(name, row, 0);
		ret.setValueAt(value, row, 1);
	}
	
}
