/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.properties;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

/**
 * Creates property sets backed by java.util.HashMap.
 * @author Pavel
 *
 */
	
public class MapPropertySet implements PropertySet {
	private Map map = new HashMap();		
	private PropertySet[] chain = {};
	
	public MapPropertySet(PropertySet[] chain) {
		if (chain!=null) {
			this.chain = chain;
		}
	}
	
	public MapPropertySet(Map source, PropertySet[] chain) {
		this(chain);
		if (source!=null) {
			map.putAll(source);
		}
	}

	public MapPropertySet() {

	}
	
	public MapPropertySet(Map source) {
		if (source!=null) {
			map.putAll(source);
		}
	}
	
	public Set getPropertyNames() {
		HashSet ret = new HashSet(map.keySet());
		for (int i=0; i<chain.length; ++i) {
			ret.addAll(chain[i].getPropertyNames());
		}
		return ret;
	}

	public PropertySet getSubset(String prefix) {
		return new PropertySubSet(this, prefix);
	}

	public boolean remove(String name) {
		return map.remove(name)!=null;
	}

	public void set(String name, Object value) {
		map.put(name, value);
	}

	public Object get(String name) {
		Object ret = map.get(name);
		if (ret!=null) {
			return ret;
		}
		
		Iterator mit = mounts.entrySet().iterator();
		while (mit.hasNext()) {
			Map.Entry mount= (Entry) mit.next();
			if (name.startsWith((String) mount.getKey())) {
				ret = ((PropertySet) mount.getValue()).get(name.substring(((String) mount.getKey()).length()));
				if (ret!=null) {
					return ret;
				}
			}
		}
		
		for (int i=0; i<chain.length; ++i)  {
			ret = chain[i].get(name);
			if (ret!=null) {
				return ret;
			}
		}
		
		return ret;
	}

	public void clear() {
		map.clear();		
	}

	private Map mounts = new HashMap();

	public void mount(String prefix, PropertySet source) {
		mounts.put(prefix, source);		
	}
	
	public void setAll(PropertySet source) {
		Iterator nit = source.getPropertyNames().iterator();
		while (nit.hasNext()) {
			String pName = (String) nit.next();
			set(pName, source.get(pName));
		}		
	}	
	
	/**
	 * @return true if other property set entries are equal to this property set entries. 
	 * Other Property set attribute
	 */
	public boolean compareProperties(PropertySet otherSet) {
		Set myNames = new HashSet(getPropertyNames());
		Set otherNames = new HashSet(otherSet.getPropertyNames());
		if (!myNames.equals(otherNames)) {
			return false;
		}
		
		Iterator mnit = myNames.iterator();
		while (mnit.hasNext()) {
			String pName = (String) mnit.next();
			Object myValue = get(pName);
			Object otherValue = otherSet.get(pName);
			if (myValue==null) {
				if (otherValue!=null) {
					return false;
				}
			} else if (!myValue.equals(otherValue)) {
				return false;
			}			
		}
		
		return true;
	}

	public boolean containsAll(PropertySet subSet) {
		Set myNames = new HashSet(getPropertyNames());
		Set subsetNames = new HashSet(subSet.getPropertyNames());
		if (!myNames.containsAll(subsetNames)) {
			return false;
		}
		
		Iterator snit = subsetNames.iterator();
		while (snit.hasNext()) {
			String pName = (String) snit.next();
			Object myValue = get(pName);
			Object subsetValue = subSet.get(pName);
			if (myValue==null) {
				if (subsetValue!=null) {
					return false;
				}
			} else if (!myValue.equals(subsetValue)) {
				return false;
			}			
		}
		
		return true;
	}

	public Object get(String name, Object defaultValue) {
		Object ret = get(name);
		return ret == null ? defaultValue : ret;
	}	
}

