/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.properties;

import java.util.Set;

import biz.hammurapi.config.MutableContext;

/**
 * Set of properties
 * @author Pavel
 *
 * @param <V> Value type.
 */
public interface PropertySet extends MutableContext {
	
	/**
	 * @return Names of properties in the set
	 */
	Set getPropertyNames();
	
	/**
	 * @param prefix
	 * @return Property set which operates on properties starting with given prefix.
	 */
	PropertySet getSubset(String prefix);
	
	/**
	 * Removes property from the set.
	 * @param name
	 */
	boolean remove(String name);
	
	/**
	 * Removes all entries from the property set
	 */
	void clear();
	
	/**
	 * Mounts source property set at specified prefix. E.g. if source property set has entry 'a' and prefix is 'b/' then that entry will be 
	 * available as 'b/a'. Own property set entries shall shadow mounted entries.
	 * @param prefix
	 * @param source
	 */
	void mount(String prefix, PropertySet source);
	
	/**
	 * Copies all entries from the source property set to self.
	 * @param source
	 */
	void setAll(PropertySet source);	
	
	/**
	 * @param subSet
	 * @return true if all properties of subset are present in this set with same values
	 */
	boolean containsAll(PropertySet subSet);
	
	/**
	 * @param otherSet
	 * @return true if all properties in this set are equal to properties in the other set.
	 */
	boolean compareProperties(PropertySet otherSet);
	
	/**
	 * Retrieves property.
	 */
	public Object get(String name);
	
	/**
	 * @param name Property name.
	 * @param defaultValue Default value.
	 * @return Property value or default value if property doesn't exist in the
	 * property set.
	 */
	public Object get(String name, Object defaultValue);
	
	/**
	 * Sets property.
	 * @param name
	 * @param value
	 */
	public void set(String name, Object value);
}
