package biz.hammurapi.registry;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.configx.XmlConfigFactory;

/**
 * Registry entry which groups other entries.
 * @author Pavel
 *
 */
public class Directory implements RegistryEntry {
	
	private List<RegistryEntry> children = new ArrayList<RegistryEntry>();
	private String description;
	private String name;
	private String tooltip;
	
	public Directory() {
		
	}
	
	public Directory(String name, Collection<RegistryEntry> children) {
		this.children.addAll(children);
		this.name = name;
	}
	
	public Directory(String name, RegistryEntry ... children) {
		this(name, Arrays.asList(children));
	}
	
	public void addEntry(RegistryEntry child) {
		children.add(child);
	}
	
	public void addEntry(URL childConfigUrl) throws ConfigurationException {
		RegistryEntry child = (RegistryEntry) XmlConfigFactory.create(childConfigUrl, getClass().getClassLoader());
		children.add(child);
	}

	public List<RegistryEntry> getChildren(
			Class<?>[] componentTypes,
			Class<?> factoryTypes, 
			Set<String> filterTokens) {

		List<RegistryEntry> ret = new ArrayList<RegistryEntry>();
		for (RegistryEntry child: children) {
			if (child.match(componentTypes, factoryTypes, filterTokens)) {
				ret.add(child);
			}
		}
		return ret;
	}
	
	private static final Class<?>[] EMPTY_CLASS_ARRAY = {};

	public Class<?>[] getComponentTypes() {
		return EMPTY_CLASS_ARRAY;
	}

	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}

	public Class<?>[] getFactoryTypes() {
		return EMPTY_CLASS_ARRAY;
	}

	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public String getTooltip() {
		return tooltip;
	}
	
	public void setTooltip(String tooltip) {
		this.tooltip = tooltip;
	}

	public boolean hasChildren(
			Class<?>[] componentTypes,
			Class<?> factoryTypes, 
			Set<String> filterTokens) {
		
		for (RegistryEntry child: children) {
			if (child.match(componentTypes, factoryTypes, filterTokens)) {
				return true;
			}
			
			if (child.hasChildren(componentTypes, factoryTypes, filterTokens)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Directory never matches.
	 */
	public boolean match(
			Class<?>[] componentTypes, 
			Class<?> factoryTypes,
			Set<String> filterTokens) {
		return false;
	}

	/**
	 * Directory cannot be instantiated.
	 * @return null
	 */
	public <T> FactoryConfigurator<T> newFactory(Class<T> factoryClass) {
		return null;
	}

	// TODO implement merge() to load Registry service from config files.
}
