package biz.hammurapi.registry;

import java.awt.Component;
import java.util.List;
import java.util.Map;

/**
 * Implementations of this interface are used to configure and instantiate
 * component factories.
 * @author Pavel
 */
public interface FactoryConfigurator<T> {
	
	/**
	 * Configures factory. 
	 * @param context Configuration context. Some configuration parameters can be
	 * taken from the context to pre-populate or hide property pages if information
	 * is already available.
	 * @return If conifguration requires user interaction then a list of property 
	 * pages is returned. These pages are then present to the user in a wizard.
	 */
	List<Component> configure(Map<?, ?> context);
	
	// TODO - Means to make factory configurator generic, i.e. not dependent on 
	// Swing. It can be web-based for example, configured component will result 
	// in a jar file download. The jar file will have required classes plus
	// configuration stored in META-INF/config/...
	
	/**
	 * Instantiates factory for component/service. Factories aren't supposed to 
	 * implement any particular interface. E.g. CodeSnippet 'factory' creates 
	 * a fragment of code for component instantiation/configuration. 
	 * @return Configured component factory.
	 */
	T createFactory();
}
