package biz.hammurapi.sql.syntax;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Arrays;

import biz.hammurapi.sql.Variant;

/**
 * In
 * @author Pavel
 *
 */
public class In implements StatementFragment, Serializable {
	
	private static int hashCode(Object[] array) {
		final int prime = 31;
		if (array == null)
			return 0;
		int result = 1;
		for (int index = 0; index < array.length; index++) {
			result = prime * result
					+ (array[index] == null ? 0 : array[index].hashCode());
		}
		return result;
	}

	private Variant[] variants;
	private String columnName;
	
	public In(String columnName, boolean[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, byte[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, char[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, double[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, float[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, int[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, long[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, String[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, Object[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, Object[] values, int sqlType) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public In(String columnName, short[] values) {
		variants = new Variant[values.length];
		for (int i=0; i<values.length; ++i) {
			variants[i] = new Variant(values[i]);
		}
		this.columnName = columnName;
	}

	public String toSqlString() {
		StringBuffer ret = new StringBuffer(columnName);
		ret.append(" IN (");
		for (int i=0; i<variants.length; ++i) {
			if (i>0) {
				ret.append(", ");				
			}
			ret.append("?");
		}
		ret.append(")");
		return ret.toString();
	}

	public int parameterize(PreparedStatement ps, int idx) throws SQLException {
		for (int i=0; i<variants.length; ++i) {
			idx=variants[i].parameterize(ps, idx);
		}
		return idx;
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((columnName == null) ? 0 : columnName.hashCode());
		result = prime * result + In.hashCode(variants);
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final In other = (In) obj;
		if (columnName == null) {
			if (other.columnName != null)
				return false;
		} else if (!columnName.equals(other.columnName))
			return false;
		if (!Arrays.equals(variants, other.variants))
			return false;
		return true;
	}

}
