package biz.hammurapi.sql.syntax;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class Or implements StatementFragment, Serializable {
	
	private List fragments = new ArrayList();

	public Or(List fragments) {
		this.fragments.addAll(fragments);
	}

	public Or(StatementFragment[] fragments) {
		this.fragments.addAll(Arrays.asList(fragments));
	}
	
	public Or() {
		// Empty OR.
	}
	
	public void add(StatementFragment fragment) {
		fragments.add(fragment);
	}
	
	public int parameterize(PreparedStatement statement, int idx) throws SQLException {
		Iterator it = fragments.iterator();
		while (it.hasNext()) {
			idx = ((StatementFragment) it.next()).parameterize(statement, idx);
		}
		
		return idx;
	}

	public String toSqlString() {
		StringBuffer ret = new StringBuffer();
		Iterator it = fragments.iterator();
		while (it.hasNext()) {
			ret.append(((StatementFragment) it.next()).toSqlString());
			if (it.hasNext()) {
				ret.append(" OR ");
			}
			
		}
		
		return ret.toString();
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((fragments == null) ? 0 : fragments.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Or other = (Or) obj;
		if (fragments == null) {
			if (other.fragments != null)
				return false;
		} else if (!fragments.equals(other.fragments))
			return false;
		return true;
	}
}
