package biz.hammurapi.sql.syntax;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class SelectModifier implements StatementFragment {
	
	private StatementFragment where;
	private StatementFragment groupBy;
	private List orderBy = new ArrayList();
	private StatementFragment having;
	private Integer limit;
	private Integer offset;
	
	public void setWhere(StatementFragment where) {
		this.where = where;
	}
	
	public void setGroupBy(StatementFragment groupBy) {
		this.groupBy = groupBy;
	}
	
	public void addOrderBy(String column, boolean desc) {
		orderBy.add(desc ? column + " DESC" : column);
	}
	
	public void addOrderBy(String column) {
		orderBy.add(column);
	}
	
	public void setHaving(StatementFragment having) {
		this.having = having;
	}
	
	public void setLimit(int limit) {
		this.limit = new Integer(limit);
	}
	
	public void setOffset(int offset) {
		this.offset = new Integer(offset);
	}

	public String toSqlString() {
		StringBuffer ret = new StringBuffer();
		if (where!=null) {
			ret.append(" WHERE ");
			ret.append(where.toSqlString());
		}
		if (groupBy!=null) {
			ret.append(" GROUP BY ");
			ret.append(groupBy.toSqlString());
		}
		if (having!=null) {
			ret.append(" HAVING ");
			ret.append(having.toSqlString());
		}
		if (!orderBy.isEmpty()) {
			ret.append(" ORDER BY ");
			Iterator it = orderBy.iterator();
			while (it.hasNext()) {
				ret.append(it.next());
				if (it.hasNext()) {
					ret.append(", ");
				}
			}
		}
		if (limit!=null) {
			ret.append(" LIMIT ");
			ret.append(limit);
			if (offset!=null) {
				ret.append(" OFFSET ");
				ret.append(offset);
			}
		}
		
		return ret.toString();
	}

	public int parameterize(PreparedStatement ps, int idx) throws SQLException {
		if (where!=null) {
			idx = where.parameterize(ps, idx);
		}
		if (groupBy!=null) {
			idx = groupBy.parameterize(ps, idx);
		}
		if (having!=null) {
			idx = having.parameterize(ps, idx);
		}
		return idx;
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((groupBy == null) ? 0 : groupBy.hashCode());
		result = prime * result + ((having == null) ? 0 : having.hashCode());
		result = prime * result + ((limit == null) ? 0 : limit.hashCode());
		result = prime * result + ((offset == null) ? 0 : offset.hashCode());
		result = prime * result + ((orderBy == null) ? 0 : orderBy.hashCode());
		result = prime * result + ((where == null) ? 0 : where.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final SelectModifier other = (SelectModifier) obj;
		if (groupBy == null) {
			if (other.groupBy != null)
				return false;
		} else if (!groupBy.equals(other.groupBy))
			return false;
		if (having == null) {
			if (other.having != null)
				return false;
		} else if (!having.equals(other.having))
			return false;
		if (limit == null) {
			if (other.limit != null)
				return false;
		} else if (!limit.equals(other.limit))
			return false;
		if (offset == null) {
			if (other.offset != null)
				return false;
		} else if (!offset.equals(other.offset))
			return false;
		if (orderBy == null) {
			if (other.orderBy != null)
				return false;
		} else if (!orderBy.equals(other.orderBy))
			return false;
		if (where == null) {
			if (other.where != null)
				return false;
		} else if (!where.equals(other.where))
			return false;
		return true;
	}

}
