package biz.hammurapi.sql.syntax;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import biz.hammurapi.sql.Parameterizer;

public class StatementBuilder implements StatementFragment, Serializable {
	
	private StringBuffer sql = new StringBuffer();
	private List parameterizers = new ArrayList();
	
	public StatementBuilder(String sql) {
		this(sql, null);
	}
	
	public StatementBuilder(String sql, Parameterizer parameterizer) {
		if (sql!=null) {
			this.sql.append(sql);
		}
		
		if (parameterizer!=null) {
			parameterizers.add(parameterizer);
		}		
	}
	
	public StatementBuilder() {
		
	}
	
	public StatementBuilder(StatementFragment fragment) {
		this(fragment==null ? null : fragment.toSqlString(), fragment);
	}

	public void append(String sql) {
		if (sql!=null) {
			if (this.sql.length()>0) {
				this.sql.append(" ");
			}
			this.sql.append(sql);
		}
	}
	
	public void append(String sql, Parameterizer parameterizer) {
		append(sql);
		if (parameterizer!=null) {
			parameterizers.add(parameterizer);
		}		
	}
	
	public void append(StatementFragment fragment) {
		if (fragment!=null) {
			append(fragment.toSqlString(), fragment);
		}
	}
	
	public String toSqlString() {
		return sql.toString();
	}

	public int parameterize(PreparedStatement ps, int idx) throws SQLException {
		Iterator it = parameterizers.iterator();
		while (it.hasNext()) {
			idx=((Parameterizer) it.next()).parameterize(ps, idx);
		}
		return idx;
	}
	
	public static StatementBuilder assemble(String sql, Parameterizer parameterizer, StatementFragment fragment) {
		StatementBuilder ret = new StatementBuilder(sql, parameterizer);
		if (fragment!=null) {
			ret.append(fragment);
		}
		return ret;
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((parameterizers == null) ? 0 : parameterizers.hashCode());
		result = prime * result + ((sql == null) ? 0 : sql.toString().hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final StatementBuilder other = (StatementBuilder) obj;
		if (parameterizers == null) {
			if (other.parameterizers != null)
				return false;
		} else if (!parameterizers.equals(other.parameterizers))
			return false;
		if (sql == null) {
			if (other.sql != null)
				return false;
		} else if (other.sql==null || !sql.toString().equals(other.sql.toString()))
			return false;
		return true;
	}

}
