/*
  * hgcommons 9
 * Hammurapi Group Common Library 
 * Copyright (C) 2003  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://www.hammurapi.biz/hammurapi-biz/ef/xmenu/hammurapi-group/products/products/hgcommons/index.html
 * e-Mail: support@hammurapi.biz
  */
package biz.hammurapi.swing;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeNode;

/**
 * @author Pavel Vlasov
 *
 * @version $Revision: 1.1 $
 */
public class ThrowableVisualizer {
	
	
	public Visualizable convert(final Throwable th) {
		return new Visualizable() {

			public TreeNode toTreeNode(TreeNode parent, String title) {
				return new LazyTreeNodeTableVisualizable(parent, title) {

					protected List loadChildren() {
						ArrayList ret = new ArrayList();
						StackTraceElement[] st = th.getStackTrace();
					    for (int i=0; i<st.length; i++) {
					    	TreeNode ev=ThrowableVisualizer.this.toTreeNode(this, "frame", st[i]);
					    	if (ev!=null) {
					    		ret.add(ev);
					    	}
					    }							
						return ret;
					}

					public TableModel toTable() {
						DefaultTableModel tm=new DefaultTableModel(th.getCause()==null ? 2 : 3, 2);
						tm.setColumnIdentifiers(new String[] {"Property", "Value"});
						tm.setValueAt("Class", 0, 0);
						tm.setValueAt(th.getClass().getName(), 0, 1);
						tm.setValueAt("Message", 1, 0);
						tm.setValueAt(th.getMessage(), 1, 1);
						
						if (th.getCause()!=null) {
							tm.setValueAt("Cause", 2, 0);
							tm.setValueAt(th.getCause(), 2, 1);					
						}
						
						return tm;
					}
					
				};
			}
			
		};
	}

	private TreeNode toTreeNode(final TreeNode parent, final String title, final StackTraceElement element) {
		return new LazyTreeNodeTableVisualizable(parent, title) {

			protected List loadChildren() {
				return new ArrayList();
			}

			public TableModel toTable() {
				DefaultTableModel tm=new DefaultTableModel(5, 2);
				tm.setColumnIdentifiers(new String[] {"Property", "Value"});
				tm.setValueAt("Class", 0, 0);
				tm.setValueAt(element.getClassName(), 0, 1);
				
				tm.setValueAt("File", 1, 0);
				tm.setValueAt(element.getFileName(), 1, 1);
				
				tm.setValueAt("Method", 2, 0);
				tm.setValueAt(element.getMethodName(), 2, 1);
				
				tm.setValueAt("Line", 3, 0);
				tm.setValueAt(String.valueOf(element.getLineNumber()), 3, 1);
				
				tm.setValueAt("Native", 4, 0);
				tm.setValueAt(element.isNativeMethod() ? "yes" : "no", 4, 1);
				
				
				return tm;
			}
			
		};
	}
}
